package com.wyattjmiller.classscheduleapp

import android.content.Intent
import android.content.SharedPreferences
import android.graphics.Color
import androidx.appcompat.app.AppCompatActivity
import android.os.Bundle
import android.preference.PreferenceManager
import android.view.*
import android.widget.TextView
import android.widget.Toast
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView

class CourseActivity : AppCompatActivity(), CourseDialogFragment.onCourseEnteredListener {
    private lateinit var mDialog: CourseDialogFragment
    private var mCourseListRecyclerView: RecyclerView? = null
    private var mCourseAdapter: CourseAdapter? = null
    private lateinit var mCourseDb: CourseDatabase
    private lateinit var mSelectedCourse: Course
    private var mActionMode: ActionMode? = null
    private var mSharedPrefs: SharedPreferences? = null
    private var mDarkTheme = false
    private var mSelectedCoursePosition = RecyclerView.NO_POSITION

    // when the app first starts
    override fun onCreate(savedInstanceState: Bundle?) {
        mSharedPrefs = PreferenceManager.getDefaultSharedPreferences(this)
        mDarkTheme = mSharedPrefs!!.getBoolean(SettingsFragment.PREFERENCE_THEME, false)

        if (mDarkTheme) {
            setTheme(R.style.DarkTheme)
        }

        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_course)

        mCourseListRecyclerView = findViewById(R.id.courseRecyclerView)
        mCourseDb = CourseDatabase.getInstance(applicationContext)
        mCourseListRecyclerView = findViewById(R.id.courseRecyclerView)
        val layoutManager: RecyclerView.LayoutManager = LinearLayoutManager(applicationContext)
        mCourseListRecyclerView!!.layoutManager = layoutManager

        mCourseAdapter = CourseAdapter(loadCourses())
        mCourseListRecyclerView!!.adapter = mCourseAdapter
    }

    override fun onCreateOptionsMenu(menu: Menu?): Boolean {
        val inflater = menuInflater
        inflater.inflate(R.menu.course_menu, menu)
        return true
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        // handle item selection
        return when (item.itemId) {
            R.id.settings -> {
                // starts the settings fragment
                // dummy snackbar for filler until I can get settings done
                val intent = Intent(this, SettingsActivity::class.java)
                startActivity(intent)
                //Snackbar.make(findViewById(android.R.id.content), "*proceeds to make raspberry noises*", 5000).show()
                true
            }
            else -> super.onOptionsItemSelected(item)
        }
    }

    override fun onCourseEntered(course: Course) {
        // Returns subject entered in the SubjectDialogFragment dialog
        if (course != null) {
            if (course.id == 0) {
                if (mCourseDb.addCourse(course)) {
                    mCourseAdapter?.addCourse(course)
                    //Snackbar.make(this@CourseActivity, "$course entered!", 5000).show()
                } else {
                    val message = resources.getString("Already exists!".toInt(), course)
                    Toast.makeText(this@CourseActivity, message, Toast.LENGTH_SHORT).show()
                }
            } else {
                if (course.id != 0) {
                    mCourseDb.updateCourse(course)
                    mCourseAdapter?.updateCourse(course)
                } else {
                    // ??
                }
            }
        }
    }

    // click the FAB for days
    fun addCourseClick(view: View) {
        // brings up dialog to add course
        // before that, though, we have pass the course context so we know what has to happen to the dialog
        mDialog = CourseDialogFragment()
        mDialog.getContext(CourseContext.ADD, mCourseDb)
        mDialog.show(supportFragmentManager, "")
    }

    private fun loadCourses() : MutableList<Course> {
        return mCourseDb.getCourses()
    }

    private inner class CourseHolder(inflater: LayoutInflater, parent: ViewGroup?) : RecyclerView.ViewHolder(inflater.inflate(R.layout.recycler_view_items, parent, false)), View.OnClickListener, View.OnLongClickListener {
        private var mCourse: Course? = null
        private val mTextView: TextView

        fun bind(course: Course) {
            mCourse = course
            mTextView.text = course.name
            mTextView.setBackgroundColor(Color.MAGENTA)
        }

        override fun onClick(view: View) {
            mDialog = CourseDialogFragment()
            mDialog.getContext(CourseContext.VIEW, mCourseDb)
            mDialog.getCourseId(mCourse!!.id)
            mDialog.show(supportFragmentManager, "")
        }

        override fun onLongClick(view: View?): Boolean {
            if (mActionMode != null) {
                return false
            }

            mSelectedCourse = mCourse!!
            mSelectedCoursePosition = adapterPosition

            // Re-bind the selected item
            mCourseAdapter?.notifyItemChanged(mSelectedCoursePosition)

            // Show the CAB
            mActionMode = this@CourseActivity.startActionMode(mActionModeCallback)!!
            return true
        }

        private val mActionModeCallback: ActionMode.Callback = object : ActionMode.Callback {
            override fun onCreateActionMode(mode: ActionMode, menu: Menu?): Boolean {
                // Provide context menu for CAB
                val inflater = mode.menuInflater
                inflater.inflate(R.menu.context_menu, menu)
                return true
            }

            override fun onPrepareActionMode(mode: ActionMode?, menu: Menu?): Boolean {
                return false
            }

            override fun onActionItemClicked(mode: ActionMode, item: MenuItem): Boolean {
                // Process action item selection
                return when (item.itemId) {
                    R.id.delete -> {
                        // Delete from the database and remove from the RecyclerView
                        mCourseDb.deleteCourse(mSelectedCourse)
                        mCourseAdapter?.removeCourse(mSelectedCourse)

                        // Close the CAB
                        mode.finish()
                        true
                    }
                    R.id.edit -> {
                        // up comes a dialog box editing the desired list item
                        mCourseAdapter?.takeCourseIndex()
                        mDialog = CourseDialogFragment()
                        mDialog.getContext(CourseContext.EDIT, mCourseDb)
                        mDialog.getCourseId(mSelectedCourse.id)
                        mDialog.show(supportFragmentManager, "")
                        true
                    }
                    else -> false
                }
            }

            override fun onDestroyActionMode(mode: ActionMode) {
                mActionMode = null

                // CAB closing, need to deselect item if not deleted
                mCourseAdapter?.notifyItemChanged(mSelectedCoursePosition)
                mSelectedCoursePosition = RecyclerView.NO_POSITION
            }
        }

        init {
            itemView.setOnClickListener(this)
            itemView.setOnLongClickListener(this)
            mTextView = itemView.findViewById(R.id.courseTextView)
        }
    }

    private inner class CourseAdapter(private val mCourseList: MutableList<Course>) : RecyclerView.Adapter<CourseHolder>() {
        private var mOldCourseIndex = 0

        override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): CourseHolder {
            val layoutInflater = LayoutInflater.from(applicationContext)
            return CourseHolder(layoutInflater, parent)
        }

        override fun onBindViewHolder(holder: CourseHolder, position: Int) {
            holder.bind(mCourseList[position])
        }

        override fun getItemCount(): Int {
            return mCourseList.size
        }

        fun takeCourseIndex() {
            val index = mSelectedCoursePosition
            mOldCourseIndex = index
        }

        fun removeCourse(course: Course?) {
            // Find subject in the list
            val index = mCourseList.indexOf(course)
            if (index >= 0) {
                // Remove the subject
                mCourseList.removeAt(index)

                // Notify adapter of subject removal
                notifyItemRemoved(index)
            }
        }

        fun addCourse(course: Course?) {
            // Add the new subject at the beginning of the list
            mCourseList.add(0, course!!)

            // Notify the adapter that item was added to the beginning of the list
            notifyItemInserted(0)

            // Scroll to the top
            mCourseListRecyclerView?.scrollToPosition(0)
        }

        fun updateCourse(course: Course?) {
            mCourseList.removeAt(mOldCourseIndex)
            mCourseList.add(0, course!!)
            notifyItemChanged(0)
            mCourseListRecyclerView?.scrollToPosition(0)
        }
    }
}