package com.wyattjmiller.classscheduleapp

import android.content.ContentValues
import android.content.Context
import android.database.sqlite.SQLiteDatabase
import android.database.sqlite.SQLiteOpenHelper
import java.util.*

class CourseDatabase(context: Context?) : SQLiteOpenHelper(context, DATABASE_NAME, null, VERSION) {
    override fun onCreate(db: SQLiteDatabase) {
        db.execSQL(
            "create table " + CourseTable.TABLE + " (" +
                    CourseTable.COL_ID + " integer primary key autoincrement, " +
                    CourseTable.COL_NAME + " text, " +
                    CourseTable.COL_DESCRIPTION + " text, " +
                    CourseTable.COL_LOCATION + " text, " +
                    CourseTable.COL_DAYOFWEEK + " text, " +
                    CourseTable.COL_TIME + " text, " +
                    CourseTable.COL_INSTRUCTOR + " text) "
        )
    }

    override fun onUpgrade(db: SQLiteDatabase, oldVersion: Int, newVersion: Int) {
        db.execSQL("drop table if exists " + CourseTable.TABLE)
        onCreate(db)
    }

    override fun onOpen(db: SQLiteDatabase) {
        super.onOpen(db)
        if (!db.isReadOnly) {
            // Enable foreign key constraints
            db.setForeignKeyConstraintsEnabled(true)
        }
    }

    fun getCourses(): MutableList<Course> {
        val courses: MutableList<Course> = ArrayList<Course>()
        val db = this.readableDatabase
        val sql =
            "select * from " + CourseTable.TABLE
        val cursor = db.rawQuery(sql, null)

        if (cursor.moveToFirst()) {
            do {
                val course = Course()
                course.id = cursor.getInt(0)
                course.name = cursor.getString(1)
                course.description = cursor.getString(2)
                course.location = cursor.getString(3)
                course.dayofweek = cursor.getString(4)
                course.time = cursor.getString(5)
                course.instructor = cursor.getString(6)
                courses.add(course)
            } while (cursor.moveToNext())
        }

        cursor.close()
        return courses
    }

    fun getCourse(courseid: Int): Course {
        var course = Course()
        val db = this.readableDatabase
        val sql =
            "select * from " + CourseTable.TABLE +
                    " where " + CourseTable.COL_ID + " = ?"
        val cursor = db.rawQuery(sql, arrayOf(java.lang.Float.toString(courseid.toFloat())))

        if (cursor.moveToFirst()) {
            course.id = cursor.getInt(0)
            course.name = cursor.getString(1)
            course.description = cursor.getString(2)
            course.location = cursor.getString(3)
            course.dayofweek = cursor.getString(4)
            course.time = cursor.getString(5)
            course.instructor = cursor.getString(6)
        }

        cursor.close()
        return course
    }

    fun addCourse(course: Course): Boolean {
        val db = writableDatabase
        val values = ContentValues()
        values.put(CourseTable.COL_NAME, course.name)
        values.put(CourseTable.COL_DESCRIPTION, course.description)
        values.put(CourseTable.COL_LOCATION, course.location)
        values.put(CourseTable.COL_DAYOFWEEK, course.dayofweek)
        values.put(CourseTable.COL_TIME, course.time)
        values.put(CourseTable.COL_INSTRUCTOR, course.instructor)
        val id = db.insert(CourseTable.TABLE, null, values)
        course.id = id.toInt()
        return id != -1L
    }

    fun updateCourse(course: Course) {
        val db = writableDatabase
        val values = ContentValues()
        values.put(CourseTable.COL_NAME, course.name)
        values.put(CourseTable.COL_DESCRIPTION, course.description)
        values.put(CourseTable.COL_LOCATION, course.location)
        values.put(CourseTable.COL_DAYOFWEEK, course.dayofweek)
        values.put(CourseTable.COL_TIME, course.time)
        values.put(CourseTable.COL_INSTRUCTOR, course.instructor)
        db.update(CourseTable.TABLE, values, CourseTable.COL_ID + " = " + course.id, null)
    }

    fun deleteCourse(course: Course) {
        val db = writableDatabase
        db.delete(CourseTable.TABLE, CourseTable.COL_NAME + " = ?", arrayOf(course.name))
    }

    companion object {
        private const val DATABASE_NAME = "courses.db"
        private const val VERSION = 1
        private var mCourseDb: CourseDatabase? = null

        fun getInstance(context: Context?): CourseDatabase {
            if (mCourseDb == null) {
                mCourseDb = CourseDatabase(context)
            }
            return mCourseDb!!
        }
    }
}


