package com.wyattjmiller.classscheduleapp

import android.app.Dialog
import android.content.Context
import android.os.Bundle
import android.text.Editable
import android.view.View
import android.widget.Button
import android.widget.EditText
import android.widget.TextView
import android.widget.Toast
import androidx.appcompat.app.AlertDialog
import androidx.fragment.app.DialogFragment

class CourseDialogFragment : DialogFragment() {
    private lateinit var mTextViewTitle: TextView
    private lateinit var mEditTextCourseName: EditText
    private lateinit var mEditTextCourseInstructor: EditText
    private lateinit var mEditTextCourseLocation: EditText
    private lateinit var mEditTextDayOfWeek: EditText
    private lateinit var mEditTextCourseTime: EditText
    private lateinit var mEditTextCourseDescription: EditText
    private lateinit var mContext: CourseContext
    private lateinit var mCourseDb: CourseDatabase

    private lateinit var mCourseNameView: TextView
    private lateinit var mCourseLocationView: TextView
    private lateinit var mCourseTimeView: TextView
    private lateinit var mCourseDateView: TextView
    private lateinit var mCourseInstructor: TextView
    private lateinit var mCourseDescriptionView: TextView


    private var mCourse: Course = Course()

    interface onCourseEnteredListener {
        fun onCourseEntered(course: Course)
    }

    private lateinit var mDoneButton: Button
    private var mListener: onCourseEnteredListener? = null

    override fun onCreateDialog(savedInstanceState: Bundle?): Dialog {
        val builder = AlertDialog.Builder(this.activity!!)
        val dialogView = activity!!.layoutInflater.inflate(R.layout.dialog_course, null)

        mDoneButton = dialogView.findViewById(R.id.buttonFinish)
        mTextViewTitle = dialogView.findViewById(R.id.textViewTitle)
        mEditTextCourseName = dialogView.findViewById(R.id.editTextCourseName)
        mEditTextCourseLocation = dialogView.findViewById(R.id.editTextCourseLocation)
        mEditTextCourseInstructor = dialogView.findViewById(R.id.editTextCourseInstructor)
        mEditTextDayOfWeek = dialogView.findViewById(R.id.editTextCourseDayOfWeek)
        mEditTextCourseTime = dialogView.findViewById(R.id.editTextCourseTime)
        mEditTextCourseDescription = dialogView.findViewById(R.id.editTextCourseDescription)

        mCourseNameView = dialogView.findViewById(R.id.textviewCourseName2)
        mCourseLocationView = dialogView.findViewById(R.id.textViewCourseLocation2)
        mCourseDateView = dialogView.findViewById(R.id.textViewCourseDayOfWeek2)
        mCourseTimeView = dialogView.findViewById(R.id.textViewCourseTime2)
        mCourseInstructor = dialogView.findViewById(R.id.textViewCourseInstructor2)
        mCourseDescriptionView = dialogView.findViewById(R.id.textViewCourseDescription2)

        mCourseNameView.visibility = View.GONE
        mCourseLocationView.visibility = View.GONE
        mCourseDateView.visibility = View.GONE
        mCourseTimeView.visibility = View.GONE
        mCourseInstructor.visibility = View.GONE
        mCourseDescriptionView.visibility = View.GONE

        when (mContext) {
            CourseContext.ADD -> {
                mTextViewTitle.text = "Add Course"
            }
            CourseContext.EDIT -> {
                var course = mCourseDb.getCourse(mCourse.id)

                mTextViewTitle.text = "Edit Course"
                mEditTextCourseName.setText(course.name)
                mEditTextCourseLocation.setText(course.location)
                mEditTextCourseInstructor.setText(course.instructor)
                mEditTextDayOfWeek.setText(course.dayofweek)
                mEditTextCourseTime.setText(course.time)
                mEditTextCourseDescription.setText(course.description)
            }
            CourseContext.VIEW -> {
                val course = mCourseDb.getCourse(mCourse.id)

                mEditTextCourseName.visibility = View.INVISIBLE
                mEditTextCourseLocation.visibility = View.INVISIBLE
                mEditTextCourseInstructor.visibility = View.INVISIBLE
                mEditTextDayOfWeek.visibility = View.INVISIBLE
                mEditTextCourseTime.visibility = View.INVISIBLE
                mEditTextCourseDescription.visibility = View.INVISIBLE

                mCourseNameView.visibility = View.VISIBLE
                mCourseLocationView.visibility = View.VISIBLE
                mCourseDateView.visibility = View.VISIBLE
                mCourseTimeView.visibility = View.VISIBLE
                mCourseInstructor.visibility = View.VISIBLE
                mCourseDescriptionView.visibility = View.VISIBLE

                mTextViewTitle.text = "View Course"
                mCourseNameView.text = course.name
                mCourseLocationView.text = course.location
                mCourseDateView.text = course.dayofweek
                mCourseTimeView.text = course.time
                mCourseInstructor.text = course.instructor
                mCourseDescriptionView.text = course.description
            }
        }

        builder.setView(dialogView)

        mDoneButton.setOnClickListener {
            when (mContext) {
                CourseContext.ADD -> {
                    mCourse.name = mEditTextCourseName.text.toString()
                    mCourse.description = mEditTextCourseDescription.text.toString()
                    mCourse.location = mEditTextCourseLocation.text.toString()
                    mCourse.dayofweek = mEditTextDayOfWeek.text.toString()
                    mCourse.time = mEditTextCourseTime.text.toString()
                    mCourse.instructor = mEditTextCourseInstructor.text.toString()

                    val valid = this.validityCheck()
                    if (valid) {
                        mListener?.onCourseEntered(mCourse)
                        dismiss()
                    }
                }
                CourseContext.EDIT -> {
                    mCourse.name = mEditTextCourseName.text.toString()
                    mCourse.description = mEditTextCourseDescription.text.toString()
                    mCourse.location = mEditTextCourseLocation.text.toString()
                    mCourse.dayofweek = mEditTextDayOfWeek.text.toString()
                    mCourse.time = mEditTextCourseTime.text.toString()
                    mCourse.instructor = mEditTextCourseInstructor.text.toString()

                    val valid = this.validityCheck()
                    if (valid) {
                        mListener?.onCourseEntered(mCourse)
                        dismiss()
                    }
                }
                CourseContext.VIEW -> {
                    dismiss()
                }
            }
        }

        return builder.create()
    }

    fun getContext(context: CourseContext, database: CourseDatabase) {
        mContext = context
        mCourseDb = database
    }

    fun getCourseId(courseId: Int) {
        mCourse.id = courseId
    }

    override fun onAttach(context: Context) {
        super.onAttach(context)
        mListener = context as onCourseEnteredListener
    }

    private fun validityCheck() : Boolean {
        return if (mEditTextCourseName.text.toString() == "" || mEditTextCourseLocation.text.toString() == "" || mEditTextDayOfWeek.text.toString() == ""
            || mEditTextCourseTime.text.toString() == "" || mEditTextCourseInstructor.text.toString() == "" || mEditTextCourseDescription.text.toString() == "") {
            Toast.makeText(this.activity!!, "Please fill in all of the fields", Toast.LENGTH_LONG).show()
            false
        } else {
            true
        }
    }

    companion object {
        const val EXTRA_COURSE = "com.wyattjmiller.classscheduleapp.course"
    }

}